<?php
/*
* 2022 smartbill.ro
*
*  @author smartbill.ro <contact@smartbill.ro>
*  @copyright  2022 smartbill.ro
*/

if ( ! defined( '_PS_VERSION_' ) ) {
	exit;
}

define( 'SMRT_PLUGIN_VERSION', '3.1.3' );

define( 'SMRT_DATABASE_INVOICE_STATUS_DRAFT', 0 );
define( 'SMRT_DATABASE_INVOICE_STATUS_FINAL', 1 );
define( 'SMRT_DOCUMENT_TYPE_INVOICE', 0 );
define( 'SMRT_DOCUMENT_TYPE_ESTIMATE', 1 );
define( 'SMRT_VAT_VALUE_FOR_PLATFORM', 'PrestaShop' );


require_once __DIR__ . '/classes/WebserviceSpecificManagementSyncstock.php';

// All fields are set in this class
require_once __DIR__ . '/classes/SmartbillSettings.php';

// Generic utility functions
require_once __DIR__ . '/classes/SmartbillUtils.php';

// SmartBill Cloud REST Client
require_once __DIR__ . '/classes/smartbill-cloud-rest-client.php';

// SmartBill data logger
require_once __DIR__ . '/classes/SmartbillDataLogger.php';

class Smartbill extends Module {

	protected $_html       = '';
	protected $_postErrors = array();
	protected $_tab = null;

	public $email;
	public $token;
	public $cif;

	var $orders_table      = 'orders';
	var $smrt_order_fields = array(
		'smartbill_document',
	);

	public function __construct() {
		$this->name                   = 'smartbill';
		$this->tab                    = 'billing_invoicing';
		$this->version                = SMRT_PLUGIN_VERSION;
		$this->ps_versions_compliancy = array(
			'min' => '1.7.0.0',
			'max' => '8.0.2',
		);
		$this->author                 = 'smartbill.ro';
		$this->is_eu_compatible       = 1;

		$this->utils = new SmartBillUtils();

		$this->currencies      = true;
		$this->currencies_mode = 'checkbox';

		$config = Configuration::getMultiple( SmartBillSettings::$auth_fields );
		if ( ! empty( $config['SMARTBILL_EMAIL'] ) ) {
			$this->email = $config['SMARTBILL_EMAIL'];
		}
		if ( ! empty( $config['SMARTBILL_API_TOKEN'] ) ) {
			$this->token = $config['SMARTBILL_API_TOKEN'];
		}
		if ( ! empty( $config['SMARTBILL_CIF'] ) ) {
			$this->cif = $config['SMARTBILL_CIF'];
		}

		$this->bootstrap = true;
		parent::__construct();
		$this->displayName      = $this->trans( 'SmartBill', array(), 'Modules.Smartbill.Admin' );
		$this->description      = $this->trans( 'Allows you to issue invoices in SmartBill Facturare/Gestiune.', array(), 'Modules.Smartbill.Admin' );
		$this->confirmUninstall = $this->trans( 'Are you sure about removing these details?', array(), 'Modules.Smartbill.Admin' );
		if ( ! isset( $this->email ) || ! isset( $this->token ) || ! isset( $this->cif ) ) {
			$this->warning = $this->trans( 'Account email, token and CIF must be configured before using this module.', array(), 'Modules.Smartbill.Admin' );
		}
	}

	public function install() {
		// If the "SmartBillSideMenu" tab does not exist yet, create it
		if (is_null($this->_tab) ) {
			$tab             = new Tab();
			$tab->class_name = 'SmartbillTab';
			$tab->module     = $this->name;
			$tab->id_parent  = (int) Tab::getIdFromClassName( 'AdminTools' );
			foreach ( Language::getLanguages( false ) as $lang ) {
				$tab->name[ (int) $lang['id_lang'] ] = $this->displayName;
			}
			if ( ! $tab->save() ) {
				return $this->_abortInstall( $this->trans( 'Unable to create the "Smartbill" tab', array(), 'Modules.Smartbill.Admin' ) );
			}
			$this->_tab=$tab;
		}

		return (
			parent::install() &&
			$this->_installSql() &&
			$this->_add_invoice_columns() &&
			$this->registerHook( 'displayOrderDetail' ) &&
			$this->registerHook( 'displayAdminOrderRight' ) &&
			$this->registerHook( 'addWebserviceResources' ) &&
			$this->registerHook( 'displayBackOfficeHeader' ) &&
			$this->registerHook( 'displayAdminOrderMainBottom' ) &&
			$this->registerHook( 'actionValidateOrder' ) );

	}

	public function hookDisplayOrderDetail( $params ) {
		if ( Configuration::get( 'SMARTBILL_PUBLIC_INVOICE' ) ) {
			$invoice_url = Db::getInstance()->executeS( 'SELECT id_order, smartbill_public_invoice FROM ' . _DB_PREFIX_ . 'orders WHERE id_order=' . $params['order']->id );
			if ( ! empty( $invoice_url ) ) {
				$invoice_url = $invoice_url[0]['smartbill_public_invoice'];
				if ( ! empty( $invoice_url ) ) {
					return '<div class="box">
                        <h4>' . $this->trans( 'Factura', array(), 'Modules.Smartbill.Admin' ) . '</h4>
                        <a target="_blank" href="' . $invoice_url . '">' . $this->trans( 'Vizualizeaza factura', array(), 'Modules.Smartbill.Admin' ) . '</a>
                    </div>';
				}
				return '';
			}
			return '';
		}
		return '';

	}

	public function hookActionValidateOrder( $params ) {
		$options = SmartBillSettings::get_config_fields_values();
		if ( $options['SMARTBILL_AUTOMATICALLY_ISSUE_DOCUMENT'] && $options['SMARTBILL_ORDER_STATUS'] == $params['orderStatus']->id ) {
			$order_id = $params['order']->id;
			try {
				$smartbillInvoice = $this->utils->generateInvoiceData( $order_id );
				$client           = new SmartBill_Cloud_REST_Client( $options['SMARTBILL_EMAIL'], $options['SMARTBILL_API_TOKEN'] );
				$client->setPrestashopOrderId( $order_id );
				$invoiceLogger = new SmartBill_Data_Logger( $order_id );
				$client->setDataLogger( $invoiceLogger );

				if ( (int) $options['SMARTBILL_DOCUMENT_TYPE'] === SMRT_DOCUMENT_TYPE_INVOICE ) {
					$serverCall = $client->createInvoiceWithDocumentAddress( $smartbillInvoice );
				} elseif ( (int) $options['SMARTBILL_DOCUMENT_TYPE'] === SMRT_DOCUMENT_TYPE_ESTIMATE ) {
					$serverCall = $client->createProformaWithDocumentAddress( $smartbillInvoice );
				} else {
					throw new \Exception( Context::getContext()->getTranslator()->trans( 'Tipul de document emis este invalid.', array(), 'Modules.Smartbill.Admin' ) );
				}

				if ( $serverCall['errorText'] ) {
					$return['status']  = false;
					$return['message'] = $serverCall['message'];
					$return['error']   = $serverCall['errorText'];
					$return['orderId'] = $order_id;
				} else {
					$return['status'] = true;
					if ( isset( $serverCall['number'] ) && ( $serverCall['number'] ) ) {
						$return['message'] = Context::getContext()->getTranslator()->trans( 'Documentul a fost emis cu succes: ', array(), 'Modules.Smartbill.Admin' ) . $serverCall['message'] . $serverCall['series'] . ' ' . $serverCall['number'] . '.';
						$invoiceLogger->set_data( $order_id, 'smartbill_invoice_id', $serverCall['number'] )
						->set_data( $order_id, 'smartbill_series', $serverCall['series'] )
						->set_data( $order_id, 'smartbill_headers', $serverCall['get_headers'] )
						->set_data( $order_id, 'smartbill_document_url', $serverCall['documentUrl'] )
						->set_data( $order_id, 'smartbill_status', SMRT_DATABASE_INVOICE_STATUS_FINAL )
						->save( $order_id );

					} else {
						if ( isset( $serverCall['series'] ) ) {
							$invoiceLogger->set_data( $order_id, 'smartbill_series', $serverCall['series'] );
						}
						$invoiceLogger
						->set_data( $order_id, 'smartbill_document_url', $serverCall['documentUrl'] )
						->set_data( $order_id, 'smartbill_headers', $serverCall['get_headers'] )
						->set_data( $order_id, 'smartbill_status', SMRT_DATABASE_INVOICE_STATUS_DRAFT )
						->save( $order_id );

						$return['message'] = Context::getContext()->getTranslator()->trans( 'Operatiunea s-a desfasurat cu succes: ', array(), 'Modules.Smartbill.Admin' ) . $serverCall['message'];
					}
					$return['number']  = $serverCall['number'];
					$return['series']  = $serverCall['series'];
					$return['status']  = true;
					$return['headers'] = $serverCall['get_headers'];
					// Return json result
					$return['documentViewUrl'] = $serverCall['documentViewUrl'];
					$return['documentUrl']     = str_replace( '/editare/', '/vizualizare/', $serverCall['documentUrl'] );
				}

				Db::getInstance()->execute( 'UPDATE  ' . _DB_PREFIX_ . "orders SET `smartbill_document` = '{$return['documentUrl']}' WHERE id_order = {$order_id}" );
				Db::getInstance()->execute( 'UPDATE  ' . _DB_PREFIX_ . "orders SET `smartbill_public_invoice` = '{$return['documentViewUrl']}' WHERE id_order = {$order_id}" );
			} catch ( Exception $e ) {
				$return['error']   = $e->getMessage();
				$return['message'] = $e->getMessage();
				$return['status']  = false;
				$return['orderId'] = $order_id;
			}
		}

	}

	public function hookAddWebserviceResources() {
		return array(
			'syncstock' => array(
				'description'         => 'Sync stocks',
				'specific_management' => true,
			),
		);
	}

	public function uninstall() {

		// Remove menu tab
		if ( $id_tab = Tab::getIdFromClassName( 'Smartbill' ) ) {
			$tab = new Tab( (int) $id_tab );
			$tab->delete();
		}

		if ( ! Configuration::deleteByName( 'SMARTBILL_EMAIL' )
				|| ! Configuration::deleteByName( 'SMARTBILL_API_TOKEN' )
				|| ! Configuration::deleteByName( 'SMARTBILL_CIF' )
				|| ! parent::uninstall()
				|| ! $this->_unInstallSql() ) {
			return false;
		}
		return true;
	}

	/**
	 * Add smartbill sql table changes
	 *
	 * @return boolean
	 */
	protected function _installSql() {
		$sqlInstall = 'ALTER TABLE ' . _DB_PREFIX_ . 'orders '
				. 'ADD `smartbill_invoice_log` LONGTEXT NULL';

		$returnSql = Db::getInstance()->execute( $sqlInstall );

		return $returnSql;
	}

	/**
	 * Add smartbill sql permanent table changes
	 *
	 * @return true
	 */
	protected function _add_invoice_columns() {
		try {
			$sqlInstall = 'ALTER TABLE ' . _DB_PREFIX_ . 'orders '
					. 'ADD `smartbill_public_invoice` TEXT NULL';
			Db::getInstance()->execute( $sqlInstall );
		} catch ( Exception $e ) {
		}
		try {
			$sqlInstall = 'ALTER TABLE ' . _DB_PREFIX_ . 'orders '
					. 'ADD `smartbill_document` TEXT NULL';
			Db::getInstance()->execute( $sqlInstall );
		} catch ( Exception $e ) {
		}
		return true;
	}

	/**
	 * Remove smartbill sql table changes
	 *
	 * @return boolean
	 */
	protected function _unInstallSql() {
		$sqlInstall = 'ALTER TABLE ' . _DB_PREFIX_ . 'orders '
				. 'DROP smartbill_invoice_log';

		$returnSql = Db::getInstance()->execute( $sqlInstall );

		return $returnSql;
	}

	protected function _postValidation() {
		if ( Tools::isSubmit( 'btnSubmit' ) ) {
			$options                        = array(
				'SMARTBILL_EMAIL'     => Tools::getValue( 'SMARTBILL_EMAIL' ),
				'SMARTBILL_API_TOKEN' => Tools::getValue( 'SMARTBILL_API_TOKEN' ),
				'SMARTBILL_CIF'       => Tools::getValue( 'SMARTBILL_CIF' ),
			);
			$options['SMARTBILL_EMAIL']     = trim( $options['SMARTBILL_EMAIL'] );
			$options['SMARTBILL_API_TOKEN'] = trim( $options['SMARTBILL_API_TOKEN'] );

			$status = $this->utils->validate_connection( $options );
			if ( $status !== true ) {
				$this->_postErrors[] = $status;
			} else {
				// save auth fields
				foreach ( SmartBillSettings::$auth_fields as $field ) {
					Configuration::updateValue( $field, $options[ $field ] );
				}

				// test values from API
				$raw_vat_rates = SmartBillSettings::get_vat_rates( $options['SMARTBILL_CIF'] );
				Configuration::updateValue( 'SMARTBILL_VAT_RATES', serialize( $raw_vat_rates ) );

				$raw_um = SmartBillSettings::get_measuring_units( $options['SMARTBILL_CIF'] );
				Configuration::updateValue( 'SMARTBILL_MU', serialize( $raw_um ) );

				$raw_warehouses = SmartBillSettings::get_warehouses( $options['SMARTBILL_CIF'] );
				Configuration::updateValue( 'SMARTBILL_STOCKS', serialize( $raw_warehouses ) );

				if ( ! is_array( $raw_vat_rates ) ) {
					if ( $raw_vat_rates != 'Firma este neplatitoare de tva.' ) {
						$this->_postErrors[] = $raw_vat_rates;
					}
				}
				if ( ! is_array( $raw_um ) ) {
					$this->_postErrors[] = $raw_um;
				}
				if ( ! is_array( $raw_warehouses ) ) {
					$this->_postErrors[] = $raw_stock;
				}
			}
		}
	}

	protected function _postProcess() {
		if ( Tools::isSubmit( 'btnSubmit' ) ) {
			// Authentication settings
			$all_fields = SmartBillSettings::get_all_fields();
			foreach ( $all_fields as $field ) {
				if ( $field == 'SMARTBILL_API_TOKEN' || $field == 'SMARTBILL_EMAIL' ) {
					$trimfield = Tools::getValue( $field );
					Configuration::updateValue( $field, trim( $trimfield ) );
				} else {
					Configuration::updateValue( $field, Tools::getValue( $field ) );
				}
			}

			$file = __DIR__ . DIRECTORY_SEPARATOR . 'classes' . DIRECTORY_SEPARATOR . 'smartbill_sincronizare_stocuri.log';
			if ( Configuration::get( 'SMARTBILL_SAVE_STOCK_HISTORY' ) == true && ! file_exists( $file ) ) {
				$mylog        = fopen( $file, 'a' );
				$firstmessage = json_encode( '##Started Logging##' ) . "\n";
				fwrite( $mylog, $firstmessage );
				fclose( $mylog );
			}
			if ( file_exists( $file ) && Configuration::get( 'SMARTBILL_SAVE_STOCK_HISTORY' ) == false ) {
				unlink( $file );
				$zip = __DIR__ . DIRECTORY_SEPARATOR . 'smartbill_sincronizare_stocuri.zip';
				if ( file_exists( $zip ) ) {
					unlink( $zip );
				}
			}

			if ( Tools::getValue( 'SMARTBILL_SEND_MAIL_WITH_DOCUMENT' ) ) {
				$cc         = trim( Tools::getValue( 'SMARTBILL_SEND_MAIL_CC' ) );
				$bcc        = trim( Tools::getValue( 'SMARTBILL_SEND_MAIL_BCC' ) );
				$cc         = filter_var( $cc, FILTER_SANITIZE_EMAIL );
				$bcc        = filter_var( $bcc, FILTER_SANITIZE_EMAIL );
				$show_error = false;
				if ( ! empty( $cc ) ) {
					if ( ! filter_var( $cc, FILTER_VALIDATE_EMAIL ) ) {
						$show_error = true;
					}
				}
				if ( ! empty( $bcc ) ) {
					if ( ! filter_var( $bcc, FILTER_VALIDATE_EMAIL ) ) {
						$show_error = true;
					}
				}

				if ( $show_error ) {
						$this->_html .= $this->displayError( $this->trans( 'Email-ul introdus in sectiunea Setari emitere documente nu este valid.', array(), 'Admin.Global' ) );

				}
			}
		}
		$this->_html .= $this->displayConfirmation( $this->trans( 'Settings updated', array(), 'Admin.Global' ) );
	}

	public function getContent() {
		if ( Tools::isSubmit( 'btnSubmit' ) ) {
			$this->_postValidation();
			if ( ! count( $this->_postErrors ) ) {
				$this->_postProcess();
			} else {
				foreach ( $this->_postErrors as $err ) {
					$this->_html .= $this->displayError( $err );
				}
			}
			if ( Configuration::get( 'SMARTBILL_UM' ) == false || Configuration::get( 'SMARTBILL_UM' ) == 'no_value' ) {
				$this->_html .= $this->displayError( 'Este necesara selectarea unei unitati de masura.' );
			}
		} else {
			$this->_html .= '<br />';
		}
		
		if(empty(Configuration::get('SMARTBILL_SHIPPING_NAME'))){
			Configuration::updateValue('SMARTBILL_SHIPPING_NAME',$this->trans( 'Transport', array(), 'Modules.Smartbill.Admin' ));
		}

		if(empty(Configuration::get('SMARTBILL_COUPON_TEXT'))){
			Configuration::updateValue('SMARTBILL_COUPON_TEXT',$this->trans( 'Discount (#nume_cupon#)', array(), 'Modules.Smartbill.Admin' ));
		}

		$this->_html .= $this->renderForm();

		// end

		return $this->_html;
	}

	public function checkCurrency( $cart ) {
		$currency_order    = new Currency( $cart->id_currency );
		$currencies_module = $this->getCurrency( $cart->id_currency );

		if ( is_array( $currencies_module ) ) {
			foreach ( $currencies_module as $currency_module ) {
				if ( $currency_order->id == $currency_module['id_currency'] ) {
					return true;
				}
			}
		}
		return false;
	}

	// insert custom css and javascript
	public function hookDisplayBackOfficeHeader( $params ) {
		if ( ! Module::isEnabled( $this->name ) ) {
			return false;
		}

		if ( $this->context->controller->controller_name == 'AdminOrders' ) {

			$raw_smrt_docs = Db::getInstance()->executeS( 'SELECT id_order, smartbill_document, smartbill_invoice_log FROM ' . _DB_PREFIX_ . 'orders' );
			$smrt_docs     = array();
			if ( $raw_smrt_docs ) {
				foreach ( $raw_smrt_docs as $doc ) {
					if ( ! empty( $doc['smartbill_document'] ) ) {
						$details = @unserialize( $doc['smartbill_invoice_log'] );
						if ( isset( $details['received_data'] ) ) {
							$received_data = json_decode( $details['received_data'], true );
							if ( is_array( $received_data ) ) {
								krsort( $received_data );
								$received_data                 = reset( $received_data );
								$smrt_docs[ $doc['id_order'] ] = array(
									'smartbill_document' => $doc['smartbill_document'],
									'series'             => $received_data['series'],
									'number'             => $received_data['number'],
								);
							}
						} else {
							$smrt_docs[ $doc['id_order'] ] = array(
								'smartbill_document' => $doc['smartbill_document'],
								'series'             => 'Factura',
								'number'             => '',
							);
						}
					}
				}
			}
			$smrt_docs = array( 'smrt_docs' => $smrt_docs );
			Media::addJsDef( $smrt_docs );

			$id_order                      = (int) Tools::getValue( 'id_order' );
			$smartbill_settings            = array();
			$smartbill_settings['post_id'] = $id_order;

			if ( Configuration::get( 'SMARTBILL_ORDER_STATUS' ) ) {
				$smartbill_settings['order_status'] = Configuration::get( 'SMARTBILL_ORDER_STATUS' );
			}
			if ( Configuration::get( 'SMARTBILL_AUTOMATICALLY_ISSUE_DOCUMENT' ) ) {
				$smartbill_settings['automatically_issue_document'] = Configuration::get( 'SMARTBILL_AUTOMATICALLY_ISSUE_DOCUMENT' );
			}
			$smartbill_settings = array( 'smartbill' => $smartbill_settings );
			Media::addJsDef( $smartbill_settings );
			$urlOrigin = $this->context->link->getAdminLink( 'SmartbillTab' ) . '&configure=' . $this->name . '&ajax=true&id_order=';
			$urlOrigin = array( 'ajax_url' => $urlOrigin );
			Media::addJsDef( $urlOrigin );
		}

		if ( Tools::getValue( 'configure' ) === 'smartbill' ) {
			$urlOrigin = $this->context->link->getAdminLink( 'SmartbillTab' ) . '&configure=' . $this->name . '&ajax=true';
			$urlOrigin = array( 'ajax_url' => $urlOrigin );
			Media::addJsDef( $urlOrigin );
		}

		$this->context->controller->addJQuery();
		$this->context->controller->addJS( $this->_path . 'views/js/smartbill.js' );
		$this->context->controller->addCSS( $this->_path . 'views/css/smartbill.css' );
		$this->context->controller->addJS( $this->_path . 'views/js/sweetalert.js' );
		$this->context->controller->addCSS( $this->_path . 'views/css/sweetalert.css' );
	}

	// fields for the settings page
	public function renderForm() {
		if(false === Configuration::get('SMARTBILL_SHOW_ORDER_MENTION')){
			Configuration::updateValue( 'SMARTBILL_SHOW_ORDER_MENTION','Comanda online #nr_comanda_online#' );
		}

		if(false === Configuration::get('SMARTBILL_INCLUDE_SHIPPING')){
			Configuration::updateValue( 'SMARTBILL_INCLUDE_SHIPPING', true );
		}

		// get vat rates
		$raw_vat_rates = unserialize( Configuration::get( 'SMARTBILL_VAT_RATES' ) );

		$vat_rates = array();

		// Set full name to select option
		if ( is_array( $raw_vat_rates ) ) {
			foreach ( $raw_vat_rates as $k => $v ) {
				if("Preluata din Prestashop"==$v['name']){
					$vat_rates[] = array(
						'value' => $k,
						'label' => $v['name'],
					);
					continue;
				}
				$vat_rates[] = array(
					'value' => $k,
					'label' => $v['percentage'] . ( trim( $v['percentage'] ) == '' ? '' : '%' ) . ' - ' . $v['name'],
				);
			}
			
		} else {
			$vat_rates = array(
				array(
					'value' => '',
					'label' => $raw_vat_rates,
				),
			);
		}

		// get measuring units

		if ( ! Configuration::get( 'SMARTBILL_MU' ) ) {
			$raw_mu = SmartBillSettings::get_measuring_units();
			Configuration::updateValue( 'SMARTBILL_MU', serialize( $raw_mu ) );
		} else {
			$raw_mu = unserialize( Configuration::get( 'SMARTBILL_MU' ) );
		}

		$mu[] = array(
			'value' => 'no_value',
			'label' => $this->trans( 'Alegeti unitatea de masura', array(), 'Modules.Smartbill.Admin' ),
		);

		$mu[] = array(
			'value' => 'preluata-din-smartbill',
			'label' => $this->trans( 'Preluata din nomenclatorul SmartBill', array(), 'Modules.Smartbill.Admin' ),
		);
		if ( is_array( $raw_mu ) ) {
			foreach ( $raw_mu as $unit ) {
				if ( $unit != 'no_value' && $unit != 'preluata-din-smartbill' ) {
					$mu[] = array(
						'value' => $unit,
						'label' => $unit,
					);
				}
			}
		} else {
			$mu = array(
				array(
					'value' => '',
					'label' => $raw_mu,
				),
			);
		}
		if ( ! Configuration::get( 'SMARTBILL_UM' ) ) {
			Configuration::updateValue( 'SMARTBILL_UM', 'no_value' );
		}

		// get stock
		if ( ! Configuration::get( 'SMARTBILL_STOCKS' ) ) {
			$raw_stock = SmartBillSettings::get_stock();
			Configuration::updateValue( 'SMARTBILL_STOCKS', serialize( $raw_stock ) );
		} else {
			$raw_stock = unserialize( Configuration::get( 'SMARTBILL_STOCKS' ) );
		}

		$stock = array(
			array(
				'value' => '',
				'label' => $this->trans( 'Nu', array(), 'Modules.Smartbill.Admin' ),
			),
		);
		if ( is_array( $raw_stock ) ) {
			foreach ( $raw_stock as $k => $v ) {
				$stock[] = array(
					'value' => $k,
					'label' => $v,
				);
			}
		} else {
			$stock = array(
				array(
					'value' => '',
					'label' => $raw_stock,
				),
			);
		}
		$stock2 = array(
			array(
				'value' => '',
				'label' => $this->trans( 'Alege gestiune', array(), 'Modules.Smartbill.Admin' ),
			),
		);
		if ( is_array( $raw_stock ) ) {
			foreach ( $raw_stock as $k => $v ) {
				$stock2[] = array(
					'value' => $k,
					'label' => $v,
				);
			}
		} else {
			$stock2 = array(
				array(
					'value' => '',
					'label' => $raw_stock,
				),
			);
		}
		// get order statuses
		$orderStates      = new OrderState();
		$orderStates      = $orderStates->getOrderStates( 1 );
		$order_statuses   = array();
		$order_statuses[] = array(
			'value' => '0',
			'label' => 'Alege statusul',
		);

		foreach ( $orderStates as $orderState ) {
			$order_statuses[] = array(
				'value' => $orderState['id_order_state'],
				'label' => $orderState['name'],
			);
		}

		// get document series
		$invoice_series = $estimate_series = array();

		$raw_series = SmartBillSettings::get_series_settings();
		if ( is_array( $raw_series ) ) {
			foreach ( $raw_series as $value ) {
				if ( $value['type'] == 'f' ) {
					$invoice_series[] = array(
						'label' => $value['name'],
						'value' => $value['name'],
					);
				}
				if ( $value['type'] == 'p' ) {
					$estimate_series[] = array(
						'label' => $value['name'],
						'value' => $value['name'],
					);
				}
			}
		} else {
			$invoice_series  = array(
				array(
					'value' => '',
					'label' => $raw_series,
				),
			);
			$estimate_series = $invoice_series;
		}

		$fields_form = array(
			'form' => array(
				'legend' => array(
					'title' => $this->trans( 'SmartBill versiunea ' . SMRT_PLUGIN_VERSION . ' - Autentificare', array(), 'Modules.Smartbill.Admin' ),
					'icon'  => 'icon-envelope',
				),
				'input'  => array(
					array(
						'type'     => 'text',
						'label'    => $this->trans( 'Email utilizator', array(), 'Modules.Smartbill.Admin' ),
						'name'     => 'SMARTBILL_EMAIL',
						'required' => true,
					),
					array(
						'type'     => 'text',
						'label'    => $this->trans( 'Token', array(), 'Modules.Smartbill.Admin' ),
						'name'     => 'SMARTBILL_API_TOKEN',
						'desc'     => $this->trans( 'Poti obtine token-ul din contul', array(), 'Modules.Smartbill.Admin' ) . '<a href="https://cloud.smartbill.ro/core/integrari/" target="_blank"> ' . $this->trans( 'SmartBill > Contul Meu > Integrari', array(), 'Modules.Smartbill.Admin' ) . '</a>' . $this->trans( ', zona API. ', array(), 'Modules.Smartbill.Admin' ),
						'required' => true,
					),
					array(
						'type'     => 'text',
						'label'    => $this->trans( 'Cod fiscal (CIF)', array(), 'Modules.Smartbill.Admin' ),
						'name'     => 'SMARTBILL_CIF',
						'required' => true,
					),
				),
				'submit' => array(
					'title' => $this->trans( 'Save', array(), 'Admin.Actions' ),
				),
			),
		);

		$fields_form_vat_settings = array(
			'form' => array(
				'legend' => array(
					'title' => $this->trans( 'Setari TVA', array(), 'Modules.Smartbill.Admin' ),
					'icon'  => 'icon-cogs',
				),
				'input'  => array(
					array(
						'type'    => 'select',
						'label'   => $this->trans( 'Cota TVA produse', array(), 'Modules.Smartbill.Admin' ),
						'hint'    => $this->trans( 'Ce cota TVA se va aplica produselor pe documentul emis in SmartBill', array(), 'Modules.Smartbill.Admin' ),
						'name'    => 'SMARTBILL_PRICES_VAT',
						'options' => array(
							'query' => $vat_rates,
							'id'    => 'value',
							'name'  => 'label',
						),
					),
					array(
						'type'    => 'select',
						'label'   => $this->trans( 'Cota TVA transport', array(), 'Modules.Smartbill.Admin' ),
						'name'    => 'SMARTBILL_TRANSPORT_VAT',
						'options' => array(
							'query' => $vat_rates,
							'id'    => 'value',
							'name'  => 'label',
						),
					),
					array(
						'type'    => 'switch',
						'label'   => $this->trans( 'Adaugare TVA la incasare', array(), 'Modules.Smartbill.Admin' ),
						'hint'    => $this->trans( 'Daca vrei ca pe factura sa fie afisata mentiunea "TVA la incasare".', array(), 'Modules.Smartbill.Admin' ),
						'name'    => 'SMARTBILL_USE_PAYMENT_TAX',
						'is_bool' => true,
						'values'  => array(
							array(
								'value' => true,
								'id'    => 1,
							),
							array(
								'value' => false,
								'id'    => 0,
							),
						),
					),
				),
				'submit' => array(
					'title' => $this->trans( 'Save', array(), 'Admin.Actions' ),
				),
			),
		);

		$fields_form_document_settings = array(
			'form' => array(
				'legend' => array(
					'title' => $this->trans( 'Setari emitere documente', array(), 'Modules.Smartbill.Admin' ),
					'icon'  => 'icon-cogs',
				),
				'input'  => array(
					array(
						'type'    => 'select',
						'label'   => $this->trans( 'CIF utilizat', array(), 'Modules.Smartbill.Admin' ),
						'hint'    => $this->trans( 'In cazul', array(), 'Modules.Smartbill.Admin' ) . ' <a href="https://ajutor.smartbill.ro/article/245-cif-intracomunitar">' . $this->trans( 'CIF-ului intracomunitar', array(), 'Modules.Smartbill.Admin' ) . '</a>, ' . $this->trans( 'acesta va fi preluat din', array(), 'Modules.Smartbill.Admin' ) . ' <a target="_blank" href="https://cloud.smartbill.ro/core/configurare/date-firma/"><strong>' . $this->trans( 'SmartBill > Configurare', array(), 'Modules.Smartbill.Admin' ) . '</strong></a> ' . $this->trans( 'si va fi prezent doar pe facturile emise catre clienti din afara Romaniei.', array(), 'Modules.Smartbill.Admin' ),
						'name'    => 'USE_INTRA_CIF',
						'options' => array(
							'query' => SmartBillSettings::get_cifs(),
							'id'    => 'value',
							'name'  => 'label',
						),
					),
					array(
						'type'    => 'select',
						'label'   => $this->trans( 'Tipul de document emis in SmartBill', array(), 'Modules.Smartbill.Admin' ),
						'name'    => 'SMARTBILL_DOCUMENT_TYPE',
						'options' => array(
							'query' => SmartBillSettings::$document_types,
							'id'    => 'value',
							'name'  => 'label',
						),
					),
					array(
						'type'    => 'select',
						'label'   => $this->trans( 'Serie implicita factura', array(), 'Modules.Smartbill.Admin' ),
						'name'    => 'SMARTBILL_INVOICE_SERIES',
						'options' => array(
							'query' => $invoice_series,
							'id'    => 'value',
							'name'  => 'label',
						),
					),
					array(
						'type'    => 'select',
						'label'   => $this->trans( 'Serie implicita proforma', array(), 'Modules.Smartbill.Admin' ),
						'name'    => 'SMARTBILL_ESTIMATE_SERIES',
						'options' => array(
							'query' => $estimate_series,
							'id'    => 'value',
							'name'  => 'label',
						),
					),

					array(
						'type'    => 'select',
						'label'   => $this->trans( 'Data emiterii documentului', array(), 'Modules.Smartbill.Admin' ),
						'name'    => 'SMARTBILL_DOCUMENT_DATE',
						'options' => array(
							'query' => [
								['value' => 0, 'label' => $this->trans('Data curenta', array(), 'Modules.Smartbill.Admin' )],
								['value' => 1, 'label' => $this->trans('Data crearii comenzii', array(), 'Modules.Smartbill.Admin' )]
							],
							'id'    => 'value',
							'name'  => 'label',
						),
					),

					array(
						'type'    => 'switch',
						'label'   => $this->trans( 'Emite automat documentul', array(), 'Modules.Smartbill.Admin' ),
						'hint'    => $this->trans( 'Documentul va fi emis automat in SmartBill la modificarea statusului comenzii', array(), 'Modules.Smartbill.Admin' ),
						'name'    => 'SMARTBILL_AUTOMATICALLY_ISSUE_DOCUMENT',
						'is_bool' => true,
						'values'  => array(
							array(
								'value' => true,
								'id'    => 1,
							),
							array(
								'value' => false,
								'id'    => 0,
							),
						),
					),
					array(
						'type'    => 'select',
						'label'   => $this->trans( 'Statusul comenzii', array(), 'Modules.Smartbill.Admin' ),
						'hint'    => $this->trans( 'Cand comanda va avea statusul ales, documentul va fi emis automat in SmartBill.', array(), 'Modules.Smartbill.Admin' ),
						'name'    => 'SMARTBILL_ORDER_STATUS',
						'options' => array(
							'query' => $order_statuses,
							'id'    => 'value',
							'name'  => 'label',
						),
					),
					array(
						'type'    => 'select',
						'label'   => $this->trans( 'Moneda documentului emis in SmartBill', array(), 'Modules.Smartbill.Admin' ),
						'name'    => 'SMARTBILL_BILLING_CURRENCY',
						'options' => array(
							'query' => SmartBillSettings::get_currencies(),
							'id'    => 'value',
							'name'  => 'label',
						),
					),
					array(
						'type'    => 'select',
						'label'   => $this->trans( 'Limba documentului emis in SmartBill', array(), 'Modules.Smartbill.Admin' ),
						'name'    => 'SMARTBILL_INVOICE_LANG',
						'options' => array(
							'query' => SmartBillSettings::get_languages(),
							'id'    => 'value',
							'name'  => 'label',
						),
					),
					array(
						'type'    => 'select',
						'label'   => $this->trans( 'Descarca gestiunea', array(), 'Modules.Smartbill.Admin' ),
						'name'    => 'SMARTBILL_STOCK',
						'options' => array(
							'query' => $stock,
							'id'    => 'value',
							'name'  => 'label',
						),
					),
					
					array(
						'type'    => 'switch',
						'label'   => $this->trans( 'Comanda platita va fi facturata ca incasata', array(), 'Modules.Smartbill.Admin' ),
						'name'    => 'SMARTBILL_INVOICE_CASHING',
						'is_bool' => true,
						'values'  => array(
							array(
								'value' => true,
								'id'    => 1,
							),
							array(
								'value' => false,
								'id'    => 0,
							),
						),
					),

					array(
						'type'    => 'switch',
						'label'   => $this->trans( 'Afiseaza buton "Plateste cu cardul" pe factura', array(), 'Modules.Smartbill.Admin' ),
						'hint'    => $this->trans( 'Este necesar ca in  <a target="_blank" href="https://cloud.smartbill.ro/core/integrari/">SmartBill > Contul Meu > Integrari</a> sa fie configurat un procesator de plati ( Netopia sau EuPlatesc).', array(), 'Modules.Smartbill.Admin' ),
						'name'    => 'SMARTBILL_PAYMENT_URL',
						'is_bool' => true,
						'values'  => array(
							array(
								'value' => true,
								'id'    => 1,
							),
							array(
								'value' => false,
								'id'    => 0,
							),
						),
					),

					array(
						'type'    => 'switch',
						'label'   => $this->trans( 'Emite ciorna', array(), 'Modules.Smartbill.Admin' ),
						'name'    => 'SMARTBILL_INVOICE_IS_DRAFT',
						'is_bool' => true,
						'values'  => array(
							array(
								'value' => true,
								'id'    => 1,
							),
							array(
								'value' => false,
								'id'    => 0,
							),
						),
					),

					array(
						'type'    => 'switch',
						'label'   => $this->trans( 'Emite factura cu scadenta', array(), 'Modules.Smartbill.Admin' ),
						'name'    => 'SMARTBILL_ISSUE_WITH_DUE_DATE',
						'is_bool' => true,
						'values'  => array(
							array(
								'value' => true,
								'id'    => 1,
							),
							array(
								'value' => false,
								'id'    => 0,
							),
						),
					),

					array(
						'type'     => 'text',
						'label'    => $this->trans( 'Numar de zile pana la scadenta', array(), 'Modules.Smartbill.Admin' ),
						'disabled' => true,
						'name'     => 'SMARTBILL_DUE_DAYS',
					),

					array(
						'type'    => 'switch',
						'label'   => $this->trans( 'Afiseaza data livrarii pe document', array(), 'Modules.Smartbill.Admin' ),
						'name'    => 'SMARTBILL_ISSUE_WITH_DELIVERY_DAYS',
						'is_bool' => true,
						'values'  => array(
							array(
								'value' => true,
								'id'    => 1,
							),
							array(
								'value' => false,
								'id'    => 0,
							),
						),
					),

					array(
						'type'     => 'text',
						'label'    => $this->trans( 'Numar de zile pana la data livrarii', array(), 'Modules.Smartbill.Admin' ),
						'disabled' => true,
						'name'     => 'SMARTBILL_DELIVERY_DAYS',
					),

					array(
						'type'    => 'switch',
						'label'   => $this->trans( 'Afiseaza costul transportului pe document', array(), 'Modules.Smartbill.Admin' ),
						'name'    => 'SMARTBILL_INCLUDE_SHIPPING',
						'is_bool' => true,
						'values'  => array(
							array(
								'value' => true,
								'id'    => 1,
							),
							array(
								'value' => false,
								'id'    => 0,
							),
						),
					),

					array(
						'type'     => 'text',
						'label'    => $this->trans( 'Numele afisat pe factura', array(), 'Modules.Smartbill.Admin' ),
						'disabled' => true,
						'name'     => 'SMARTBILL_SHIPPING_NAME',
					),

					array(
						'type'    => 'select',
						'label'   => $this->trans( 'Cand pe comanda costul transportului e 0', array(), 'Modules.Smartbill.Admin' ),
						'name'    => 'SMARTBILL_FREE_SHIPPING',
						'options' => array(
							'query' => [
								['value'=>0,'label'=>$this->trans('Adauga transportul pe factura', array(), 'Modules.Smartbill.Admin' )],
								['value'=>1,'label'=>$this->trans('Nu adauga transportul pe factura', array(), 'Modules.Smartbill.Admin' )],
							],
							'id'    => 'value',
							'name'  => 'label',
						),
					),

					array(
						'type'    => 'switch',
						'label'   => $this->trans( 'Afiseaza pe factura reducerea de pret pe produs', array(), 'Modules.Smartbill.Admin' ),
						'name'    => 'SMARTBILL_PRICE_INCLUDE_DISCOUNTS',
						'is_bool' => true,
						'values'  => array(
							array(
								'value' => true,
								'id'    => 1,
							),
							array(
								'value' => false,
								'id'    => 0,
							),
						),
					),

					array(
						'type'     => 'text',
						'label'    => $this->trans( 'Numele cuponului afisat pe factura', array(), 'Modules.Smartbill.Admin' ),
						'hint'     => $this->trans( 'Pentru afisarea numelui cuponului aplicat poti introduce #nume_cupon#.', array(), 'Modules.Smartbill.Admin' ),
						'name'     => 'SMARTBILL_COUPON_TEXT',
					),
					
					array(
						'type'    => 'switch',
						'label'   => $this->trans( 'Adauga datele intocmitorului/delegatului pe factura', array(), 'Modules.Smartbill.Admin' ),
						'name'    => 'SMARTBILL_ADD_DELEGATE_DATA',
						'is_bool' => true,
						'values'  => array(
							array(
								'value' => true,
								'id'    => 1,
							),
							array(
								'value' => false,
								'id'    => 0,
							),
						),
					),

					array(
						'type'     => 'text',
						'label'    => $this->trans( 'Nume intocmitor', array(), 'Modules.Smartbill.Admin' ),
						'disabled' => true,
						'class'	   => 'smartbill-show-if-add-delegate-data-on',
						'name'     => 'SMARTBILL_ISSUER_NAME',
					),

					array(
						'type'     => 'text',
						'label'    => $this->trans( 'CNP intocmitor', array(), 'Modules.Smartbill.Admin' ),
						'disabled' => true,
						'class'	   => 'smartbill-show-if-add-delegate-data-on',
						'name'     => 'SMARTBILL_ISSUER_CNP',
					),
					
					array(
						'type'     => 'text',
						'label'    => $this->trans( 'Nume delegat', array(), 'Modules.Smartbill.Admin' ),
						'disabled' => true,
						'class'	   => 'smartbill-show-if-add-delegate-data-on',
						'name'     => 'SMARTBILL_DELEGATE_NAME',
					),
					
					array(
						'type'     => 'text',
						'label'    => $this->trans( 'Buletin', array(), 'Modules.Smartbill.Admin' ),
						'disabled' => true,
						'class'	   => 'smartbill-show-if-add-delegate-data-on',
						'name'     => 'SMARTBILL_DELEGATE_BULLETIN',
					),
					
					array(
						'type'     => 'text',
						'label'    => $this->trans( 'Auto', array(), 'Modules.Smartbill.Admin' ),
						'disabled' => true,
						'class'	   => 'smartbill-show-if-add-delegate-data-on',
						'name'     => 'SMARTBILL_DELEGATE_AUTO',
					),

					array(
						'type'     => 'textarea',
						'label'    => $this->trans( 'Mentiuni factura', array(), 'Modules.Smartbill.Admin' ),
						'hint'     => $this->trans( 'Mentinunile vor fi vizibile pe factura. Personalizeaza mentiunile cu date preluate automat de pe comanda folosind termeni variabili: #nr_comanda_online#, #tip_plata#', array(), 'Modules.Smartbill.Admin' ),
						'disabled' => false,
						'name'     => 'SMARTBILL_SHOW_ORDER_MENTION',
					),
	
					array(
						'type'     => 'textarea',
						'label'    => $this->trans( 'Observatii factura', array(), 'Modules.Smartbill.Admin' ),
						'hint'     => $this->trans( 'Observatiile vor fi vizibile doar pe raportul de facturi din SmartBill. Pentru a afisa numarul comenzii poti introduce #nr_comanda_online#.', array(), 'Modules.Smartbill.Admin' ),
						'disabled' => false,
						'name'     => 'SMARTBILL_SHOW_ORDER_OBS',
					),

					array(
						'type'    => 'switch',
						'label'   => $this->trans( 'Salveaza clientul in SmartBill', array(), 'Modules.Smartbill.Admin' ),
						'hint'    => $this->trans( 'Salvand clientul in SmartBill, vei avea datele lui disponibile pentru emiteri ulterioare', array(), 'Modules.Smartbill.Admin' ),
						'name'    => 'SMARTBILL_SAVE_CLIENT',
						'is_bool' => true,
						'values'  => array(
							array(
								'value' => true,
								'id'    => 1,
							),
							array(
								'value' => false,
								'id'    => 0,
							),
						),
					),

					array(
						'type'    => 'switch',
						'label'   => $this->trans( 'Afiseaza factura in contul clientului', array(), 'Modules.Smartbill.Admin' ),
						'hint'    => $this->trans( 'Clientul va putea vedea factura emisa in SmartBill la accesarea comenzii din contul creat pe site.', array(), 'Modules.Smartbill.Admin' ),
						'name'    => 'SMARTBILL_PUBLIC_INVOICE',
						'is_bool' => true,
						'values'  => array(
							array(
								'value' => true,
								'id'    => 1,
							),
							array(
								'value' => false,
								'id'    => 0,
							),
						),
					),

				),
				'submit' => array(
					'title' => $this->trans( 'Save', array(), 'Admin.Actions' ),
				),
			),
		);

		$fields_form_send_documments_settings = array(
			'form' => array(
				'legend' => array(
					'title' => $this->trans( 'Setari trimitere documente', array(), 'Modules.Smartbill.Admin' ),
					'icon'  => 'icon-envelope',
				),
				'input'  => array(
					array(
						'type'    => 'switch',
						'label'   => $this->trans( 'Trimite automat documentul clientului', array(), 'Modules.Smartbill.Admin' ),
						'hint'    => $this->trans( 'Documentul va fi trimis pe email clientului automat dupa facturarea comenzii', array(), 'Modules.Smartbill.Admin' ),
						'name'    => 'SMARTBILL_SEND_MAIL_WITH_DOCUMENT',
						'is_bool' => true,
						'values'  => array(
							array(
								'value' => true,
								'id'    => 1,
							),
							array(
								'value' => false,
								'id'    => 0,
							),
						),
					),
					array(
						'type'     => 'text',
						'label'    => $this->trans( 'Cc:', array(), 'Modules.Smartbill.Admin' ),
						'name'     => 'SMARTBILL_SEND_MAIL_CC',
						'required' => false,
					),
					array(
						'type'     => 'text',
						'label'    => $this->trans( 'Bcc:', array(), 'Modules.Smartbill.Admin' ),
						'name'     => 'SMARTBILL_SEND_MAIL_BCC',
						'required' => false,
						'desc'     => '<br/>' . $this->trans( 'Subiectul si mesajul email-ului trimis clientului este cel configurat in SmartBill > Configurare> Email. Factura in format PDF va fi atasata email-ului.', array(), 'Modules.Smartbill.Admin' ),

					),
				),
				'submit' => array(
					'title' => $this->trans( 'Save', array(), 'Admin.Actions' ),
				),
			),
		);

		$fields_form_product_settings = array(
			'form' => array(
				'legend' => array(
					'title' => $this->trans( 'Setari produse', array(), 'Modules.Smartbill.Admin' ),
					'icon'  => 'icon-envelope',
				),
				'input'  => array(
					array(
						'type'    => 'switch',
						'label'   => $this->trans( 'Salveaza produsul in SmartBill', array(), 'Modules.Smartbill.Admin' ),
						'name'    => 'SMARTBILL_SAVE_PRODUCT',
						'is_bool' => true,
						'values'  => array(
							array(
								'value' => true,
								'id'    => 1,
							),
							array(
								'value' => false,
								'id'    => 0,
							),
						),
					),
					array(
						'type'    => 'select',
						'label'   => $this->trans( 'Unitate de masura implicita', array(), 'Modules.Smartbill.Admin' ),
						'hint'    => $this->trans( 'Ce unitate de masura se va aplica produselor pe documentul emis in SmartBill', array(), 'Modules.Smartbill.Admin' ),
						'name'    => 'SMARTBILL_UM',
						'options' => array(
							'query' => $mu,
							'id'    => 'value',
							'name'  => 'label',
						),
					),
					array(
						'type'    => 'select',
						'label'   => $this->l( 'Tipul de cod folosit in PrestaShop' ),
						'hint'    => $this->l( 'Alege codurile pe care le folosesti in PrestaShop si care sunt echivalente cu codurile din SmartBill' ),
						'name'    => 'SMARTBILL_PRODUCT_SKU_TYPE',
						'options' => array(
							'query' => SmartBillSettings::get_sku_types(),
							'id'    => 'value',
							'name'  => 'label',
						),
					),
					array(
						'type'    => 'switch',
						'label'   => $this->trans( 'La facturare, preia denumirile produselor din SmartBill', array(), 'Modules.Smartbill.Admin' ),
						'hint'    => $this->l( 'Pe documentul emis se vor afisa denumirile produselor din SmartBill, pe baza codului de produs' ),
						'name'    => 'SMARTBILL_PRODUCT',
						'is_bool' => true,
						'values'  => array(
							array(
								'value' => true,
								'id'    => 1,
							),
							array(
								'value' => false,
								'id'    => 0,
							),
						),
					),
				),
				'submit' => array(
					'title' => $this->trans( 'Save', array(), 'Admin.Actions' ),
				),
			),
		);

		$fields_form_stock_settings = array(
			'form' => array(
				'legend' => array(
					'title' => $this->trans( 'Sincronizare stocuri', array(), 'Modules.Smartbill.Admin' ),
					'icon'  => 'icon-envelope',
				),
				'input'  => array(
					array(
						'type'    => 'switch',
						'label'   => $this->trans( 'Actualizeaza stocurile din magazinul online', array(), 'Modules.Smartbill.Admin' ),
						'name'    => 'SMARTBILL_SYNC_STOCK',
						'desc'    => $this->get_desc_sync_stock(),
						'is_bool' => true,
						'values'  => array(
							array(
								'value' => true,
								'id'    => 1,
							),
							array(
								'value' => false,
								'id'    => 0,
							),
						),
					),
					array(
						'type'    => 'select',
						'label'   => $this->trans( 'Gestiune utilizata', array(), 'Modules.Smartbill.Admin' ),
						'desc'    => $this->add_manual_sync_stock_button(),
						'name'    => 'SMARTBILL_USED_STOCK',
						'options' => array(
							'query' => $stock2,
							'id'    => 'value',
							'name'  => 'label',
						),
					),
					array(
						'type'    => 'switch',
						'label'   => $this->trans( 'Activeaza istoric sincronizare automata', array(), 'Modules.Smartbill.Admin' ),
						'name'    => 'SMARTBILL_SAVE_STOCK_HISTORY',
						'hint'    => $this->trans( 'Permite inregistrarea tuturor actiunilor referitoare la sincronizarea de stoc.', array(), 'Modules.Smartbill.Admin' ),
						'desc'    => $this->add_download_history_button(),
						'is_bool' => true,
						'values'  => array(
							array(
								'value' => true,
								'id'    => 1,
							),
							array(
								'value' => false,
								'id'    => 0,
							),
						),
					),

				),

				'submit' => array(
					'title' => $this->trans( 'Save', array(), 'Admin.Actions' ),
				),
			),
		);

		$helper                           = new HelperForm();
		$helper->show_toolbar             = false;
		$helper->table                    = $this->table;
		$lang                             = new Language( (int) Configuration::get( 'PS_LANG_DEFAULT' ) );
		$helper->default_form_language    = $lang->id;
		$helper->allow_employee_form_lang = Configuration::get( 'PS_BO_ALLOW_EMPLOYEE_FORM_LANG' ) ? : 0;
		$this->fields_form                = array();
		$helper->id                       = (int) Tools::getValue( 'id_carrier' );
		$helper->identifier               = $this->identifier;
		$helper->submit_action            = 'btnSubmit';
		$helper->currentIndex             = $this->context->link->getAdminLink( 'AdminModules', false ) . '&configure='
			. $this->name . '&tab_module=' . $this->tab . '&module_name=' . $this->name;
		$helper->token                    = Tools::getAdminTokenLite( 'AdminModules' );
		$helper->tpl_vars                 = array(
			'fields_value' => SmartBillSettings::get_raw_config_fields_values(),
			'languages'    => $this->context->controller->getLanguages(),
			'id_language'  => $this->context->language->id,
		);

		if ( empty( Configuration::get( 'SMARTBILL_CIF' ) ) ) {
			return $helper->generateForm( array( $fields_form ) );
		} elseif ( empty( Configuration::get( 'SMARTBILL_TAXES' ) ) ) {
			return $helper->generateForm( array( $fields_form, $fields_form_document_settings, $fields_form_send_documments_settings, $fields_form_product_settings, $fields_form_stock_settings ) );
		} else {
			return $helper->generateForm( array( $fields_form, $fields_form_vat_settings, $fields_form_document_settings, $fields_form_send_documments_settings, $fields_form_product_settings, $fields_form_stock_settings ) );
		}

	}
	public function hookDisplayAdminOrderMainBottom( $params ) {
		if ( ! $this->active ) {
			return;
		}

		$params['id_order'] = (int) $params['id_order'];
		$order              = 0;
		$order              = $this->getOrderSmartBillFields( $params['id_order'] );
		$urlOrigin          = $this->context->link->getAdminLink( 'SmartbillTab' ) . '&configure=' . $this->name . '&ajax=true&id_order=' . $params['id_order'] . '&action=';
		$urlExport          = $urlOrigin . 'exportDocument';
		$urlMail            = $urlOrigin . 'sendMailDocument';
		$urlView            = ! empty( $order['smartbill_document'] ) ? $order['smartbill_document'] : '';
		$urlErrors          = $urlOrigin . 'SendErrors';

		$this->html = '
        <div class="left-column" id="smartbill-invoice-panel-backoffice">
            <div class="card">
                <div class="card-header">
                    <h3 class="card-header-title">
                        FACTURARE SMARTBILL
                    </h3>
                </div>
                <div class="card-body smartbill-container">
                    ' . ( empty( $urlView ) ? '<a class="btn btn-primary smrt-ajax" style="color: white!important" data-href="' . $urlExport . '"><i class="icon-exchange"></i> ' . $this->trans( 'Emite document', array(), 'Modules.Smartbill.Admin' ) . '</a>' : '<a style="color: white!important;" class="btn btn-primary smrt-ajax reissue" data-href="' . $urlExport . '"><i class="icon-exchange"></i> ' . $this->trans( 'Reemite document', array(), 'Modules.Smartbill.Admin' ) . '</a>' ) . '
                    ' . ( ! empty( $urlView ) ? '<a class="btn btn-primary smrt-doc" href="' . $urlView . '" target="_blank"><i class="icon-file"></i> ' . $this->trans( 'Vizualizeaza in SmartBill', array(), 'Modules.Smartbill.Admin' ) . '</a>' : '' ) . '
                    ' . ( ! empty( $urlView ) ? '<a class="btn btn-primary smrt-ajax-mail" style="color: white!important"  data-href="' . $urlMail . '" ><i class="icon-file"></i> ' . $this->trans( 'Trimite documentul clientului', array(), 'Modules.Smartbill.Admin' ) . '</a>' : '' ) . '
                </div>
            </div>
        </div>';

		return $this->html;
	}

	public function hookDisplayAdminOrderRight( $params ) {
		if ( ! $this->active ) {
			return;
		}

		$params['id_order'] = (int) $params['id_order'];
		$order              = 0;
		$order              = $this->getOrderSmartBillFields( $params['id_order'] );
		$urlOrigin          = $this->context->link->getAdminLink( 'SmartbillTab' ) . '&configure=' . $this->name . '&ajax=true&id_order=' . $params['id_order'] . '&action=';
		$urlExport          = $urlOrigin . 'exportDocument';
		$urlMail            = $urlOrigin . 'sendMailDocument';
		$urlView            = ! empty( $order['smartbill_document'] ) ? $order['smartbill_document'] : '';
		$urlErrors          = $urlOrigin . 'SendErrors';

		$this->html = '
            <div class="panel smartbill-container" id="smartbill-invoice-panel-backoffice">
                <div class="panel-heading"> FACTURARE SMARTBILL</div>
                ' . ( empty( $urlView ) ? '<a class="btn btn-primary smrt-ajax" data-href="' . $urlExport . '"><i class="icon-exchange"></i> ' . $this->trans( 'Emite document', array(), 'Modules.Smartbill.Admin' ) . '</a>' : '<a class="btn btn-primary smrt-ajax reissue" data-href="' . $urlExport . '"><i class="icon-exchange"></i> ' . $this->trans( 'Reemite document', array(), 'Modules.Smartbill.Admin' ) . '</a>' ) . '
                ' . ( ! empty( $urlView ) ? '<a class="btn btn-primary smrt-doc" href="' . $urlView . '" target="_blank"><i class="icon-file"></i> ' . $this->trans( 'Vizualizeaza in SmartBill', array(), 'Modules.Smartbill.Admin' ) . '</a>' : '' ) . '
                ' . ( ! empty( $urlView ) ? '<a class="btn btn-primary smrt-ajax-mail" style="color: white!important"  data-href="' . $urlMail . '" target="_blank"><i class="icon-file"></i> ' . $this->trans( 'Trimite documentul clientului', array(), 'Modules.Smartbill.Admin' ) . '</a>' : '' ) . '
            </div>';

		return $this->html;
	}

	private function getOrderSmartBillFields( $id ) {
		$sql   = 'SELECT ' . implode( ',', $this->smrt_order_fields ) . ' FROM ' . _DB_PREFIX_ . $this->orders_table . ' WHERE id_order=' . (int) $id;
		$order = Db::getInstance()->executeS( $sql );
		if ( ! empty( $order ) ) {
			$order = $order[0];
		}

		return $order;
	}

	private function get_desc_sync_stock() {
		$key = Configuration::get('SMARTBILL_SYNK_KEY');
		
		if( true == $key && !WebserviceKey::keyExists($key) ){
			$key = false;
		}
		if( true == $key && !WebserviceKey::isKeyActive($key) ){
			$key = 'CHEIE_SERVICIU_WEB';
		}
		
		if( false == $key && true == Configuration::get('PS_WEBSERVICE')){
			$key = md5(sha1(microtime(true).random_int(10000,90000)));
			$apiAccess = new WebserviceKey();
			$apiAccess->key = $key;
			$apiAccess->description = "SmartBill sincronizare stoc";
			$apiAccess->save();
			$permissions = [
				'syncstock' => ['GET' => 1, 'POST' => 1, 'PUT' => 1, 'PATCH' => 1, 'DELETE' => 1, 'HEAD' => 1],
			];
			WebserviceKey::setPermissionForAccount($apiAccess->id, $permissions);
			Configuration::updateValue( 'SMARTBILL_SYNK_KEY', $key );
			
			$actual_link = ( isset( $_SERVER['HTTPS'] ) && $_SERVER['HTTPS'] === 'on' ? 'https' : 'http' ) . "://$_SERVER[HTTP_HOST]$_SERVER[REQUEST_URI]";
			$host_url    = substr( $actual_link, 0, strpos( $actual_link, 'admin' ) );
			return '<div class="token_details"><p id="url_container_stocks">URL: <input type="text" readonly value="' . $host_url . 'api/syncstock/?ws_key='.$key. '"/></p>
			<p>' . $this->trans( 'Acest URL va fi introdus in ', array(), 'Modules.Smartbill.Admin' ) . '<a href="https://cloud.smartbill.ro/core/integrari/" target="_blank">' . $this->trans( 'SmartBill Cloud > Contul Meu > Integrari > Sincronizare stocuri > URL.', array(), 'Modules.Smartbill.Admin' ) . '</a></p>
			<p>' . $this->trans( 'Token-ul cu care se face autentificarea in plugin-ul SmartBill va trebui introdus in ', array(), 'Modules.Smartbill.Admin' ) . '<a href="https://cloud.smartbill.ro/core/integrari/#sincronizare_stocuri" target="_blank">' . $this->trans( 'SmartBill Cloud > Contul Meu > Integrari > Sincronizare stocuri > Token autentificare.', array(), 'Modules.Smartbill.Admin' ) . '</a></p>
			<p><a style="margin-right:5px;" href="https://www.youtube.com/watch?v=2d_7j2HxaHc" target="_blank" class="btn btn-primary">' . $this->trans( 'Vezi video', array(), 'Modules.Smartbill.Admin' ) . '</a> <a href="https://ajutorgestiune.smartbill.ro/article/866-sincronizarea-stocurilor-cu-magazinul-online-cu-prestashop" target="_blank" class="btn btn-primary">' . $this->trans( 'Consulta ghid ', array(), 'Modules.Smartbill.Admin' ) . '</a></p></div>
			';
		}
		if( false == Configuration::get('PS_WEBSERVICE')){
			$actual_link = ( isset( $_SERVER['HTTPS'] ) && $_SERVER['HTTPS'] === 'on' ? 'https' : 'http' ) . "://$_SERVER[HTTP_HOST]$_SERVER[REQUEST_URI]";
			$host_url    = substr( $actual_link, 0, strpos( $actual_link, 'admin' ) );
			return '<div class="token_details"><p id="url_container_stocks">URL: <input type="text" readonly value="' . $host_url . 'api/syncstock/?ws_key=CHEIE_SERVICIU_WEB"/></p>
			<p>' . $this->trans( 'URL-ul se va genera automat dupa accesarea in Back Office->Parametri Avansati->Servicii Web si activarea si salvarea configurarii "Activeaza serviciile web Prestashop".', array(), 'Modules.Smartbill.Admin' ) . '</p>
			<p>' . $this->trans( 'Acest URL va fi introdus in ', array(), 'Modules.Smartbill.Admin' ) . '<a href="https://cloud.smartbill.ro/core/integrari/" target="_blank">' . $this->trans( 'SmartBill Cloud > Contul Meu > Integrari > Sincronizare stocuri > URL.', array(), 'Modules.Smartbill.Admin' ) . '</a></p>
			<p>' . $this->trans( 'Token-ul cu care se face autentificarea in plugin-ul SmartBill va trebui introdus in ', array(), 'Modules.Smartbill.Admin' ) . '<a href="https://cloud.smartbill.ro/core/integrari/#sincronizare_stocuri" target="_blank">' . $this->trans( 'SmartBill Cloud > Contul Meu > Integrari > Sincronizare stocuri > Token autentificare.', array(), 'Modules.Smartbill.Admin' ) . '</a></p>
			<p><a style="margin-right:5px;" href="https://www.youtube.com/watch?v=2d_7j2HxaHc" target="_blank" class="btn btn-primary">' . $this->trans( 'Vezi video', array(), 'Modules.Smartbill.Admin' ) . '</a> <a href="https://ajutorgestiune.smartbill.ro/article/866-sincronizarea-stocurilor-cu-magazinul-online-cu-prestashop" target="_blank" class="btn btn-primary">' . $this->trans( 'Consulta ghid ', array(), 'Modules.Smartbill.Admin' ) . '</a></p></div>
			';
		}

		if(true == $key){
			$actual_link = ( isset( $_SERVER['HTTPS'] ) && $_SERVER['HTTPS'] === 'on' ? 'https' : 'http' ) . "://$_SERVER[HTTP_HOST]$_SERVER[REQUEST_URI]";
			$host_url    = substr( $actual_link, 0, strpos( $actual_link, 'admin' ) );
			return '<div class="token_details"><p id="url_container_stocks">URL: <input type="text" readonly value="' . $host_url . 'api/syncstock/?ws_key='.$key. '"/></p>
			<p>' . $this->trans( 'Acest URL va fi introdus in ', array(), 'Modules.Smartbill.Admin' ) . '<a href="https://cloud.smartbill.ro/core/integrari/" target="_blank">' . $this->trans( 'SmartBill Cloud > Contul Meu > Integrari > Sincronizare stocuri > URL.', array(), 'Modules.Smartbill.Admin' ) . '</a></p>
			<p>' . $this->trans( 'Token-ul cu care se face autentificarea in plugin-ul SmartBill va trebui introdus in ', array(), 'Modules.Smartbill.Admin' ) . '<a href="https://cloud.smartbill.ro/core/integrari/#sincronizare_stocuri" target="_blank">' . $this->trans( 'SmartBill Cloud > Contul Meu > Integrari > Sincronizare stocuri > Token autentificare.', array(), 'Modules.Smartbill.Admin' ) . '</a></p>
			<p><a style="margin-right:5px;" href="https://www.youtube.com/watch?v=2d_7j2HxaHc" target="_blank" class="btn btn-primary">' . $this->trans( 'Vezi video', array(), 'Modules.Smartbill.Admin' ) . '</a> <a href="https://ajutorgestiune.smartbill.ro/article/866-sincronizarea-stocurilor-cu-magazinul-online-cu-prestashop" target="_blank" class="btn btn-primary">' . $this->trans( 'Consulta ghid ', array(), 'Modules.Smartbill.Admin' ) . '</a></p></div>
			';
		}

	}

	private function add_download_history_button() {
		$last_stock_update = Configuration::get( 'SMARTBILL_STOCK_UPDATE');
		if( !empty($last_stock_update) ){
			$last_stock_update = unserialize(Configuration::get( 'SMARTBILL_STOCK_UPDATE'));
			$last_stock_update = $last_stock_update->format('d/m/Y H:i');
		}
		if ( Configuration::get( 'SMARTBILL_SAVE_STOCK_HISTORY' ) == true ) {
		
			return '
			<input id="smartbill-download-sync-stock-history" type="button" class="btn btn-primary" value="' . $this->trans( 'Descarca istoric', array(), 'Modules.Smartbill.Admin' ) . '">
			<p>Data ultimei actualizari a stocurilor: '.$last_stock_update.'</p>
			';
		}
		return '<p>Data ultimei actualizari a stocurilor: '.$last_stock_update.'</p>';
	}

	private function add_manual_sync_stock_button(){
		return '<input id="smartbill-manually-sync-stock" type="button" class="btn btn-primary" value="' . $this->trans( 'Preia manual stocurile din SmartBill', array(), 'Modules.Smartbill.Admin' ) . '">';
	}

}
